# Copyright 2011, Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
#
# Author: Josef Skladanka <jskladan@redhat.com>

import xmlrpclib

class ResultsDBapi(object):
    def __init__(self, resultsdb_api_url):
        self._server = xmlrpclib.ServerProxy(resultsdb_api_url, allow_none = 1)

    def start_job(self, name = None, log_url = None):
        """
        Use to logically bond several testruns (e.g. for multi-package tests etc).

        Returns
            * job_id ~ identifier to use in subsequent resultsdb calls
        """
        return self._server.start_job(name, log_url)

    def end_job(self, job_id, result, log_url = None, summary = None, sync_testrun_times = False):
        """
        Stores the end_time for the job. If @param sync_testrun_times is set to True,
        then all testruns associated with the job will have their start/end times updated
        according to the job.

        Params
            * job_id ~ job identifier
            * sync_testrun_times ~ set to True in order to sync start/end times of all
                                   testruns associated with the job
        """
        return self._server.end_job(job_id, result, log_url, summary, sync_testrun_times)

    def start_testrun(self, testcase_url, keyval_pairs = None, job_id = None, log_url = None):
        """
        Use to indicate start of new test-run. New entry in ResultsDB is created
        and start_time is set to server's current time.

        Params
            * testcase_url ~ string; Testcase with this URL must exist in ResultsDB
                            (use get_testcases() to list these)
            * keyval_pairs ~ Dictionary of key-value pairs to be stored
            * job_id ~ job identifier returned from start_job method

        Returns
            * testrun_id ~ Testrun identifier (required for the subsequent XMLRPC calls)
        """
        testrun_id = self._server.start_testrun(testcase_url, job_id, keyval_pairs, log_url)
        return testrun_id

    def end_testrun(self, testrun_id, result, log_url, keyval_pairs = None, summary = None):
        """
        Use to indicate the end of testrun. Server's current time is saved as testrun's end time.
        Additionaly, information provided in params gets stored too. 

        Params
            * result ~ PASSED, FAILED, ABORTED ... if non-correct value is passed, NEEDS_INSPECTION is set 
            * log_url ~ URL pointing to logs etc. (most probably in the Autotest storage)
            * keyval_pairs ~ Dictionary of key-value pairs to be stored
            * summary ~ Short text summarizing the testrun's result.
        """

        r_val = self._server.end_testrun(testrun_id, result, log_url, keyval_pairs, summary)
        return r_val

    def start_phase(self, testrun_id, name):
        """
        Some tests may be devided in a number of phases. Phases may be nested, 
        but you always can end only the "most recently started" phase.
        Each phase has it's own result (see end_phase()), but it does not 
        directly influence the Testrun.result (i.e. you still 
        need to set result in the end_testrun()

        Params
            * name ~ Name of the phase - used for displaying in the frontends. 
        """

        return self._server.start_phase(testrun_id, name)

    def end_phase(self, testrun_id, result):
        """
        Ends the "most recently started" phase. The result does not
        directly influencethe Testrun's result

        Params
            * result ~ PASSED, FAILED, ABORTED ... if non-correct value is passed, NEEDS_INSPECTION is set
        """

        return self._server.end_phase(testrun_id, result)

    def store_keyval(self, testrun_id, keyval_pairs):
        """
        Be aware, that while storing keyval pairs, all non-string keys/values (or lists/tuples of strings
        in case of values) are skipped without further notice.

        Keyval pairs are additional data specific to the testcase like envr, repo, arch...

        These values, represented by dictionary will be parsed and stored.
        Key have to be string, value can be either string or list of strings.

        Examples
            * {"key1" : "value1"}
            * {"arch" : ["i686", "x86_64"]} is equivalent of ("arch":"i686" and "arch":"x86_64") 

        Params
            * keyval_pairs ~ Dictionary of key-value pairs to be stored. 
        """

        return self._server.store_keyval(testrun_id, keyval_pairs)

